import java.util.*;
import java.text.*;
/**
 * Clase que representa una Factura del Sistema de Facturacion
 * 
 * @author G. Cherencio
 * @version 1.0
 */
public class Factura {
    // ATRIBUTOS
    private int nro;
    private Date fecha;
    private Cliente cliente     = null;
    private boolean confirmada  = false;
    private double monto        = 0.0;
    private boolean pagada      = false;
    private List<Detalle> detalle  = new ArrayList<Detalle>();

    // CONSTRUCTORES
    /**
     * Constructor para crear objetos de la clase Factura
     */
    public Factura() {
        this(1); // asumo que el nro de la factura es 1
    }
    public Factura(int n) {
        this(n,new Date());
    }
    public Factura(int n,Date f) {
        this(n,f,new Cliente());
    }
    public Factura(int n,Date f,Cliente c) {
        setNro(n);
        setFecha(f);
        setCliente(c);
    }
    
    // GETTERS
    public int getNro()             { return nro;   }
    public Date getFecha()          { return fecha; }
    // fecha en formato dd/mm/yyyy
    public String getFechaStr()     { 
        SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
        String strFecha = sdf.format(getFecha());
        return strFecha; 
    }
    public Cliente getCliente()     { return cliente; }
    public int getCount()           { return detalle.size(); }
    public boolean isConfirmada()   { return confirmada; }
    public boolean isPagada()       { return pagada; }
    public double getMonto()        { 
        calcular();
        return monto; 
    }
    
    // SETTERS
    public void setNro(int n)           { nro = n; }
    public void setFecha(Date d)        { 
        if ( isConfirmada() ) return; // no hago nada si la factura esta confirmada
        fecha = d; 
    }
    public void setFecha(String d)      { 
        if ( isConfirmada() ) return; // no hago nada si la factura esta confirmada
        SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
        try {
            Date f = sdf.parse(d);
            setFecha(f);
        } catch(ParseException pe) {
            System.err.println("Clase Factura metodo setFecha(): Error, ["+d+"] no es una fecha en formato dd/MM/yyyy");
        }
    }
    public void setCliente(Cliente c)   { 
        if ( isConfirmada() ) return; // no hago nada si la factura esta confirmada
        cliente = c; 
    }
    
    // CUSTOM
    public void confirmar()                 {
        if ( isConfirmada() ) return; // no hago nada si la factura esta confirmada
        if ( getCliente() == null ) return; // no hay cliente asociado a la factura
        if ( getCount() <= 0 ) return; // no hay detalle de factura
        calcular(); // recalcula monto de la factura
        // sumo al saldo deudor del cliente
        double saldo = getCliente().getSaldo();
        saldo+=getMonto();
        getCliente().setSaldo(saldo);
        // descuento stock de los productos
        for(Detalle d : detalle) {
            if ( d != null ) d.getProducto().addStock(d.getCantidad()*(-1));
        }
        confirmada = true;
    } 
    public void pagar()                     {
        if ( isPagada() ) return; // no puedo pagar mas de una vez a la factura
        if ( isConfirmada() ) {
            double saldo = getCliente().getSaldo();
            saldo-=getMonto();
            getCliente().setSaldo(saldo);
        }
    } 
    private void calcular()                 {
        // recorro todos los objetos Detalle y sumarizo
        monto = 0.0;
        for(Detalle d : detalle) {
            if ( d != null ) monto+=d.getSubtotal();
        }
    }
    public void addDetalle(Detalle d)       {
        if ( !isConfirmada() ) {
            detalle.add(d);
        } else {
            System.err.println("addDetalle(): Error, factura confirmada!");
        }
    } 
    public void addDetalle(Detalle d,int p) {
        if ( p >= 0 && p < detalle.size() ) {
            if ( !isConfirmada() ) {
                detalle.add(p,d);
            } else {
                System.err.println("addDetalle(): Error, factura confirmada!");
            }
        } else {
            System.err.println("addDetalle(): Error, posicion incorrecta!");
        }
    } 
    public void delDetalle(Detalle d)       {
        if (!isConfirmada() && getCount() > 0) {
            detalle.remove(d);
        }
    } 
    public void delDetalle(int p)           {
        if ( p >= 0 && p < detalle.size() ) {
            if ( !isConfirmada() && getCount() > 0 ) {
                detalle.remove(p);
            } else {
                System.err.println("delDetalle(): Error, factura confirmada!");
            }
        } else {
            System.err.println("delDetalle(): Error, posicion incorrecta!");
        }
    }
    // SOBRE-ESCRITURA DE METODOS DE CLASE OBJECT
    public boolean equals(Object obj) {
        //System.out.println("Estoy ejecutando equals() de Cliente!");
        if ( obj == null ) return false;
        if ( obj instanceof Factura ) {
            Factura c = (Factura) obj;
            if ( c.getNro() == getNro() ) {
                return true;
            } else {
                return false;
            }
        } else return false;
    }
    public int hashCode() {
        return getNro();
    }
    /**
     * Formato de toString() para este objeto
     * [<prop>:<valor>,<prop>:<valor>,....]”
     */
    public String toString() {
        StringBuffer sb = new StringBuffer("[nro:" + getNro() + ",fecha:"+getFechaStr()+",cliente:"+getCliente()+
                ",confirmada:"+isConfirmada()+",pagada:"+isPagada()+",monto:"+getMonto()+
                ",detalle:");
        for(Detalle d : detalle) {
            if ( d != null ) sb.append(d);
        }
        sb.append(",cnt:"+getCount()+"]");
        return sb.toString();
    }
    
}
